(* The polymorphic hash function *)

value hash_univ : 'a -> int;;
        (* "hash_univ x" associates a positive integer to any value of
           any type. It is guaranteed that
                if x = y, then hash_univ x = hash_univ y. 
           hash_univ always terminates, even on cyclic structures. *)
value hash_univ_param : int -> int -> 'a -> int = 3 "hash_univ_param";;
        (* "hash_univ_param n m x" computes a hash value for x, with the
           same properties as for hash_univ. The two extra parameters n and
           m gives more precise control over hashing. Hashing performs a
           depth-first, right-to-left traversal of the structure x, stopping
           after n meaningful nodes were encountered, or m nodes, meaningful
           or not, were encountered. Meaningful nodes are: integers;
           floating-point numbers; strings; characters; booleans; and constant
           constructors. Larger values of m and n means that more
           nodes are taken into account to compute the final hash
           value, and therefore collisions are less likely to happen.
           However, hashing takes longer. Parameters m and n allow to
           trade accuracy for speed. *)
