(* Operation on lists *)

value list_length : 'a list -> int
        (* Returns the length of a list (number of elements). *)
  and prefix @ : 'a list -> 'a list -> 'a list
        (* List concatenation *)
  and rev : 'a list -> 'a list
        (* List reversal *)
  and do_list : ('a -> 'b) -> 'a list -> unit
        (* do_list f [a1; ...; an] performs f a1; f a2; ...; f an. *)
  and map : ('a -> 'b) -> 'a list -> 'b list
        (* map f [a1; ...; an] = [f a1; ...; f an] *)
  and it_list : ('a -> 'b -> 'a) -> 'a -> 'b list -> 'a
        (* it_list f a [b1; ...; bn] = f (... (f (f a b1) b2) ...) bn *)
  and list_it : ('a -> 'b -> 'b) -> 'a list -> 'b -> 'b
        (* list_it f [a1; ...; an] b = f a1 (f a2 (... (f an b) ...)) *)
  and flat_map : ('a -> 'b list) -> 'a list -> 'b list
        (* flat_map f [l1; ...; ln] = (f l1) @ (f l2) @ ... @ (f ln) *)
  and for_all : ('a -> bool) -> 'a list -> bool
        (* for_all p [a1; ...; an] = (p a1) & (p a2) & ... & (p an) *)
  and exists : ('a -> bool) -> 'a list -> bool
        (* exists p [a1; ...; an] = (p a1) or (p a2) or ... or (p an) *)
;;

value mem : 'a -> 'a list -> bool
        (* "mem a l" is true iff a is structurally equal to one element
           of list l. *)
  and memq : 'a -> 'a list -> bool
        (* "memq a l" is true iff a is physically equal to one element
           of list l. *)
  and except : 'a -> 'a list -> 'a list
        (* "except a l" returns list l where the first element structurally
           equal to a has been removed. *)
  and exceptq : 'a -> 'a list -> 'a list
        (* "except a l" returns list l where the first element physically
           equal to a has been removed. *)
  and subtract : 'a list -> 'a list -> 'a list
        (* "subtract l1 l2" returns list l1 where all elements structurally
           equal to one of the elements of l2 have been removed. *)
  and index : 'a -> 'a list -> int
        (* "index a l" returns the position of the first element of list l
           that is structurally equal to a. The head of the list has
           position 0. Raises failure "index" if a cannot be found in l. *)
;;
value assoc : 'a -> ('a * 'b) list -> 'b
        (* "assoc a l" returns the value associated with key a in the list of
           pairs l. That is,
             assoc a [ ...; (a,b); ...] = b
           if (a,b) is the leftmost binding of a in list l.
           Raises failure "find" if there is no value associated with a in the
           list. *)
  and assq :  'a -> ('a * 'b) list -> 'b
        (* Same as assoc, but uses physical equality instead of structural
           equality to compare keys. *)
  and mem_assoc : 'a -> ('a * 'b) list -> bool
        (* Same as assoc, but simply returns true if one binding exist,
           and false if no binding exist for the given key. *)
;;

